@@ -1,4 +1,4 @@
-/* $Id: iface-linux.c 1381 2007-02-04 15:01:41Z lennart $ */
+/* $Id: iface-linux.c 1421 2007-04-12 23:46:19Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -184,7 +184,7 @@ static int process_response(int wait_for_done, unsigned seq) {
     do {
         size_t bytes;
         ssize_t r;
-        char replybuf[2048];
+        char replybuf[8*1024];
         char cred_msg[CMSG_SPACE(sizeof(struct ucred))];
         struct msghdr msghdr;
         struct cmsghdr *cmsghdr;
