@@ -1,7 +1,7 @@
 #ifndef foodbuswatchgluehfoo
 #define foodbuswatchgluehfoo
 
-/* $Id: dbus-watch-glue.h 327 2005-08-15 02:42:59Z lennart $ */
+/* $Id: dbus-watch-glue.h 872 2005-10-26 01:21:30Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -26,6 +26,10 @@
 
 #include <avahi-common/watch.h>
 
+AVAHI_C_DECL_BEGIN
+
 int avahi_dbus_connection_glue(DBusConnection *c, const AvahiPoll *poll_api);
 
+AVAHI_C_DECL_END
+
 #endif
