@@ -1,7 +1,7 @@
 #ifndef fooqueryschedhfoo
 #define fooqueryschedhfoo
 
-/* $Id: query-sched.h 308 2005-08-13 21:25:09Z lennart $ */
+/* $Id: query-sched.h 954 2005-11-13 16:36:33Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -31,7 +31,8 @@ AvahiQueryScheduler *avahi_query_scheduler_new(AvahiInterface *i);
 void avahi_query_scheduler_free(AvahiQueryScheduler *s);
 void avahi_query_scheduler_clear(AvahiQueryScheduler *s);
 
-int avahi_query_scheduler_post(AvahiQueryScheduler *s, AvahiKey *key, int immediately);
+int avahi_query_scheduler_post(AvahiQueryScheduler *s, AvahiKey *key, int immediately, unsigned *ret_id);
+int avahi_query_scheduler_withdraw_by_id(AvahiQueryScheduler *s, unsigned id);
 void avahi_query_scheduler_incoming(AvahiQueryScheduler *s, AvahiKey *key);
 
 #endif
