@@ -1,7 +1,7 @@
 #ifndef foorrlisthfoo
 #define foorrlisthfoo
 
-/* $Id: rrlist.h 306 2005-08-12 21:01:28Z lennart $ */
+/* $Id: rrlist.h 954 2005-11-13 16:36:33Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -31,10 +31,12 @@ AvahiRecordList *avahi_record_list_new(void);
 void avahi_record_list_free(AvahiRecordList *l);
 void avahi_record_list_flush(AvahiRecordList *l);
 
-AvahiRecord* avahi_record_list_next(AvahiRecordList *l, int *flush_cache, int *unicast_response, int *auxiliary);
+AvahiRecord* avahi_record_list_next(AvahiRecordList *l, int *ret_flush_cache, int *ret_unicast_response, int *ret_auxiliary);
 void avahi_record_list_push(AvahiRecordList *l, AvahiRecord *r, int flush_cache, int unicast_response, int auxiliary);
 void avahi_record_list_drop(AvahiRecordList *l, AvahiRecord *r);
 
+int avahi_record_list_all_flush_cache(AvahiRecordList *l);
+
 int avahi_record_list_is_empty(AvahiRecordList *l);
 
 #endif
