@@ -1,7 +1,7 @@
 #ifndef foomainhfoo
 #define foomainhfoo
 
-/* $Id: main.h 114 2005-06-16 16:32:27Z lennart $ */
+/* $Id: main.h 1092 2006-01-20 22:56:11Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -23,7 +23,11 @@
 ***/
 
 #include <avahi-core/core.h>
+#include <avahi-common/simple-watch.h>
 
 extern AvahiServer *avahi_server;
+extern AvahiSimplePoll *simple_poll_api;
+
+extern int nss_support;
 
 #endif
