@@ -1,4 +1,4 @@
-/* $Id: avahi-resolve.c 1262 2006-08-22 02:26:11Z lennart $ */
+/* $Id: avahi-resolve.c 1466 2007-05-09 11:09:02Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -164,8 +164,8 @@ static void help(FILE *f, const char *argv0) {
             "    -v --verbose         Enable verbose mode\n"
             "    -6                   Lookup IPv6 address\n"
             "    -4                   Lookup IPv4 address\n",
-            argv0, strstr(argv0, "address") ? "[-a]" : "-a",
-            argv0, strstr(argv0, "host-name") ? "[-n]" : "-n");
+            argv0, strstr(argv0, "host-name") ? "[-n]" : "-n",
+            argv0, strstr(argv0, "address") ? "[-a]" : "-a");
 }
 
 static int parse_command_line(Config *c, const char *argv0, int argc, char *argv[]) {
@@ -186,7 +186,6 @@ static int parse_command_line(Config *c, const char *argv0, int argc, char *argv
     c->proto = AVAHI_PROTO_UNSPEC;
     c->verbose = 0;
 
-    opterr = 0;
     while ((o = getopt_long(argc, argv, "hVnav46", long_options, NULL)) >= 0) {
 
         switch(o) {
@@ -212,7 +211,6 @@ static int parse_command_line(Config *c, const char *argv0, int argc, char *argv
                 c->proto = AVAHI_PROTO_INET6;
                 break;
             default:
-                fprintf(stderr, "Invalid command line argument: %c\n", o);
                 return -1;
         }
     }
